# Generated by Django 2.0 on 2018-03-09 17:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('intonation', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='annotationcategory',
            name='item_type',
        ),
        migrations.RemoveField(
            model_name='annotationcategoryvalue',
            name='category',
        ),
        migrations.DeleteModel(
            name='AnnotationCategory',
        ),
        migrations.DeleteModel(
            name='AnnotationCategoryValue',
        ),
        migrations.DeleteModel(
            name='ItemType',
        ),
    ]
